@@ -1,7 +1,14 @@
-from .commands import CommandResult, create_commander, group, keyword, optional, var  # noqa: F401
-from .validation import validate  # noqa: F401
-from .transformation import transform  # noqa: F401
+from .commands import (  # noqa: F401
+    CommandResult,
+    create_commander,
+    group,
+    keyword,
+    optional,
+    var,
+)
 from .error import CommandError, ItemNotFoundError, ParseError  # noqa: F401
+from .transformation import transform  # noqa: F401
+from .validation import validate  # noqa: F401
 
 
-__version__ = '0.15.2'
+__version__ = "0.16.2"
