@@ -1,5 +1,5 @@
 class CommandError(RuntimeError):
-    """ raised if an error occurred while executing a command """
+    """raised if an error occurred while executing a command"""
 
     def __init__(self, message, data=None, status=1, code=None) -> None:
         super().__init__()
@@ -14,15 +14,15 @@ class CommandError(RuntimeError):
 
 
 class ParseError(CommandError):
-    """ raised in case of an invalid command """
+    """raised in case of an invalid command"""
 
 
 class ItemNotFoundError(CommandError):
-    """ raised if a variable reference was not found """
+    """raised if a variable reference was not found"""
 
 
 class InjectionError(CommandError):
-    """ raised if a dependency injection failed """
+    """raised if a dependency injection failed"""
 
 
 class ShouldThrottleException(CommandError):
@@ -33,7 +33,11 @@ class ShouldThrottleException(CommandError):
     """
 
     def __init__(self) -> None:
-        super().__init__('The command was aborted because the output interface '
-                         'was unable to write data fast enough. Please choose '
-                         'an interface with higher throughput like USB if available.',
-                         None, 1, 'KIEN.interface.data_overflow')
+        super().__init__(
+            "The command was aborted because the output interface "
+            "was unable to write data fast enough. Please choose "
+            "an interface with higher throughput like USB if available.",
+            None,
+            1,
+            "KIEN.interface.data_overflow",
+        )
